//
//
// hello.RGB.45.c
//
// RGB LED software PWM hello-world
//
// Neil Gershenfeld
// 11/10/10
//
// (c) Massachusetts Institute of Technology 2010
// This work may be reproduced, modified, distributed,
// performed, and displayed for any purpose. Copyright is
// retained and must be preserved. The work is provided
// as is; no warranty is provided, and users accept all
// liability.
//

#include <avr/io.h>
#include <util/delay.h>

#define output(directions,pin) (directions |= pin) // set port direction for output
#define input(directions,pin) (directions &= (~pin)) // set port direction for input
#define set(port,pin) (port |= pin) // set port pin
#define clear(port,pin) (port &= (~pin)) // clear port pin
#define pin_test(pins,pin) (pins & pin) // test for port pin
#define bit_test(byte,bit) (byte & (1 << bit)) // test for bit set
#define PWM_DELAY_UNIT 50
#define PWM_delay() _delay_us(PWM_DELAY_UNIT) // PWM delay

#define led_port PORTB
#define led_direction DDRB
#define red (1 << PB2)
#define green (1 << PB1)
#define blue (1 << PB0)

#define red_on() clear(led_port, red)
#define red_off() set(led_port, red)
#define green_on() clear(led_port, green)
#define green_off() set(led_port, green)
#define blue_on() clear(led_port, blue)
#define blue_off() set(led_port, blue)



#define I2C_slave_address 0x53 // ADXL345 alt address
#define I2C_delay() _delay_us(5)
#define SCL_pin (1 << PB4)
#define SCL_pins PINB
#define SCL_port PORTB
#define SCL_direction DDRB
#define SDA_pin (1 << PB3)
#define SDA_pins PINB
#define SDA_port PORTB
#define SDA_direction DDRB

void SCL_write(char bit) {
   //
   // write SCL bit
   //
   if (bit == 0) {
      output(SCL_direction,SCL_pin);
      clear(SCL_port,SCL_pin);
      }
   else {
      input(SCL_direction,SCL_pin);
      while (pin_test(SCL_pins,SCL_pin) == 0); // check for clock stretching
      }
   }

void SDA_write(char bit) {
   //
   // write SDA bit
   //
   if (bit == 0) {
      output(SDA_direction,SDA_pin);
      clear(SDA_port,SDA_pin);
      }
   else
      input(SDA_direction,SDA_pin);
   }

void I2C_init() {
   //
   // initialize I2C lines
   //
   SDA_write(1);
   SCL_write(1);
   }

char I2C_master_write_byte(unsigned char byte) {
   //
   // master write I2C byte
   //
   unsigned char bit;
   //
   // loop over bits
   //
   for (bit = 0; bit < 8; ++bit) {
      if ((byte & 0x80) == 0)
         SDA_write(0);
      else
         SDA_write(1);
      SCL_write(1);
      I2C_delay();
      SCL_write(0);
      I2C_delay();
      byte <<= 1;
      }
   //
   // check for ACK
   //
   SDA_write(1);
   SCL_write(1);
   I2C_delay();
   if (pin_test(SDA_pins,SDA_pin) != 0) {
      //
      // no ACK, return 1
      //
      return 1;
   }
   //
   // yes ACK, return 0
   //
   SCL_write(0);
   I2C_delay();
   return 0;
   }

char I2C_master_write(unsigned char* data, unsigned char nbytes, unsigned char slave_address) {
   //
   // I2C master write
   //
   unsigned char index,ret,slave_address_write;
   //
   // send start
   //
   SDA_write(0);
   I2C_delay();
   SCL_write(0);
   I2C_delay();
   //
   // send slave address
   //
   slave_address_write = slave_address << 1;
   if (I2C_master_write_byte(slave_address_write) != 0) {
      //
      // no ACK, return 1
      //
      return 1;
   }
   //
   // loop over bytes
   //
   for (index = 0; index < nbytes; ++index) {
      ret = I2C_master_write_byte(data[index]);
      if (ret != 0) {
         //
         // no ACK, return 1
         //
      break;
      }
      //
      // yes ACK, continue
      //
      }
   //
   // send stop
   //
   SCL_write(1);
   I2C_delay();
   SDA_write(1);
   I2C_delay();
   return ret;
   }

void I2C_master_read_byte(unsigned char* data, unsigned char index, unsigned char nbytes) {
   //
   // master read I2C byte
   //
   unsigned char byte,bit;
   SDA_write(1);
   byte = 0;
   //
   // loop over bits
   //
   for (bit = 0; bit < 8; ++bit)  {
      SCL_write(1);
      I2C_delay();
      if (pin_test(SDA_pins,SDA_pin) != 0)
         byte |= (1 << (7-bit));
      SCL_write(0);
      I2C_delay();
      }
   data[index] = byte;
   if (index < (nbytes-1)) {
      //
      // not done, send ACK
      //
      SDA_write(0);
      SCL_write(1);
      I2C_delay();
      SCL_write(0);
      SDA_write(1);
      I2C_delay();
      }
   else {
      //
      // done, send NACK
      //
      SDA_write(1);
      SCL_write(1);
      I2C_delay();
      SCL_write(0);
      I2C_delay();
      }
   }

char I2C_master_read(unsigned char* data, unsigned char nbytes, unsigned char slave_address) {
   //
   // I2C master read
   //
   unsigned char index,slave_address_read;
   //
   // send start
   //
   SDA_write(0);
   I2C_delay();
   SCL_write(0);
   I2C_delay();
   //
   // send slave address
   //
   slave_address_read = (slave_address << 1) + 1;
   if (I2C_master_write_byte(slave_address_read) == 1) {

   }
      //
      // no ACK, return 1
      //
      // return 1;
   //
   // loop over bytes
   //
   for (index = 0; index < nbytes; ++index)
      I2C_master_read_byte(data,index,nbytes);
   //
   // send stop
   //
   SCL_write(1);
   I2C_delay();
   SDA_write(1);
   I2C_delay();
   return 0;
}

void accel_init(unsigned char* data)
{
  I2C_init();
  data[0] = 0x2D; // POWER_CTL register
  data[1] = 8; // turn on measure bit
  ret = I2C_master_write(data,2,I2C_slave_address);
}

void read_accel_data_sample(unsigned char* data)
{
    data[0] = 0x32; // X0 register
    ret = I2C_master_write(data,1,I2C_slave_address);
    ret = I2C_master_read(data,6,I2C_slave_address);
}


void set_clock_divider_to_1()
{
  CLKPR = (1 << CLKPCE);
  CLKPR = (0 << CLKPS3) | (0 << CLKPS2) | (0 << CLKPS1) | (0 << CLKPS0);
}


void LED_pins_init()
{
  red_off();
  output(led_direction, red);
  green_off();
  output(led_direction, green);
  blue_off();
  output(led_direction, blue);
}

void pwm_delay(int units)
{
  while (0 < units)
  {
    PWM_delay();
    --units;
  }
}


void led_rgb_pwm_control(uint8_t red_value, uint8_t green_value, uint8_t blue_value)
{
    unsigned uint8_t count;
    for (count = 0; count < 255; ++count) {
      if (count <= red_value){
        red_on();
      } else{
        red_off();
      }

      if (count <= green_value){
        green_on();
      } else{
        green_off();
      }

      if (count <= blue_value){
        blue_on();
      } else{
        blue_off();
      }
      pwm_delay(1);
    }
}

uint8_t convert_acceleration_to_RGB_value(unsigned char first_byte, unsigned char second_byte)
{
  return (uint8_t) ((((uint16_t)255 * (uint16_t)second_byte) + first_byte) / 4)
}


int main(void) {
  unsigned char data[6];
  
  set_clock_divider_to_1();
  LED_pins_init();
  accel_init(data);

  while (1) {
    read_accel_data_sample(data)

    uint8_t red_value = convert_acceleration_to_RGB_value(data[0], data[1]);
    uint8_t green_value = convert_acceleration_to_RGB_value(data[2], data[3]);
    uint8_t blue_value = convert_acceleration_to_RGB_value(data[4], data[5]);

    control_led_rgb_pwm(red_value, green_value, blue_value);
  }
}
